/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.render;

import java.awt.Point;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import nl.cloudfarming.client.geoviewer.Geographical;
import nl.cloudfarming.client.geoviewer.Palette;
import org.netbeans.api.visual.model.ObjectState;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public abstract class GeographicalWidget extends Widget {

    private final Geographical geographical;
    private Palette palette;

    public GeographicalWidget(Geographical geographical, Palette palette, Scene scene) {
        super(scene);
        this.palette = palette;
        this.geographical = geographical;

        geographical.addPropertyChangeListener(new WidgetRefresher());
        setForeground(this.palette.getColorForState(getState()));
    }

    public Palette getPalette() {
        return palette;
    }

    public void setPalette(Palette palette) {
        this.palette = palette;
        setForeground(this.palette.getColorForState(getState()));
        repaint();
    }

    @Override
    protected void notifyStateChanged(ObjectState previousState, ObjectState state) {
        setForeground(this.palette.getColorForState(getState()));
        repaint();
    }

    /**
     * Determines if a widget isHit. A widget is hit when a point lays between
     * the bounds of the widget and: - The localLocation intersects the widgets
     * GeoPostition - the localLocation is within a distance of DISTANCE of
     * GeoPosition
     *
     * @param localLocation the Point which possibly hits the widget
     * @return if the widget is hit
     */
    @Override
    public boolean isHitAt(Point localLocation) {
        if (isVisible() && getBounds().contains(localLocation)) {
            //FIXME check based on envelope
            return true;
        } else {
            return false;
        }
    }
    
    protected Geographical getGeographical() {
        return geographical;
    }

    private class WidgetRefresher implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            getScene().getView().repaint();
        }
    }
}
