/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geoviewer.render;

import com.vividsolutions.jts.geom.Coordinate;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.util.List;

/**
 *
 * @author johan
 */
//TODO FP harcoded colors. What about set colors in LAF initialisation and read them from UIManager? 
public class PolygonDrawingRenderer implements DrawingRenderer {
    private final List<Coordinate> coords;

    public PolygonDrawingRenderer(List<Coordinate> coords) {
        this.coords = coords;
    }
    
    @Override
    public void render(Graphics2D g, Rectangle canvas, GeoTranslator translator) {
            
            if (coords == null || coords.isEmpty()) return;
            
            int len = coords.size();
            int xs[] = new int[len];
            int ys[] = new int[len];
            
            // TODO: calculate bounding box, 
            // no problems yet since the currently used translator doesn't need it.
            
            for (int i = 0; i < len; i++) {
                Point2D point = translator.geoToPixel(canvas, null, coords.get(i));
                xs[i] = (int) (point.getX() - canvas.x);
                ys[i] = (int) (point.getY() - canvas.y);
            }
            if (len >= 3) {
                renderShape(g, xs, ys);
            }
            if (len >= 2) {
                renderEdges(g, xs, ys);          
            }
            renderPoints(g, xs, ys);
        }
        
        private void renderPoints(Graphics2D g, int[] xs, int[]  ys) {
            int len = xs.length;
            
            g.setColor(Color.DARK_GRAY);
            for (int i = 0; i < len - 1; i++) {
                g.fillRect(xs[i] - 1, ys[i] - 1, 3, 3);
            }
            // alt. style for last point:
            g.setColor(Color.YELLOW);
            g.fillRect(xs[len - 1] - 1, ys[len - 1] - 1, 3, 3);
        }
        
        //shape outline except for connection from last to first point.
        private void renderEdges(Graphics2D g, int[] xs, int[]  ys) {
            g.setColor(Color.RED);
            g.drawPolyline(xs, ys, xs.length);
        }
        
        private void renderShape(Graphics2D g, int[] xs, int[]  ys) {
            g.setColor(new Color(255, 255, 255, 127));
            g.fillPolygon(xs, ys, xs.length);
        }    
        
}
